/* (C) 2022 YiRing, Inc. */
package com.yiring.app.domain.location;

import com.yiring.auth.domain.user.User;
import com.yiring.common.annotation.FieldMapping;
import java.io.Serializable;
import java.time.LocalTime;
import java.util.HashSet;
import java.util.Set;
import javax.persistence.*;
import lombok.*;
import lombok.experimental.FieldDefaults;
import lombok.experimental.FieldNameConstants;
import org.hibernate.annotaions.Comment;
import org.hibernate.annotations.GenericGenerator;
import org.hibernate.snowflake.SnowflakeId;

/**
 * 围栏规则
 * TODO: 规则待梳理
 *
 * @author Jim
 * @version 0.1
 * 2022/4/7 18:25
 */

@Getter
@Setter
@ToString
@Builder
@NoArgsConstructor
@AllArgsConstructor
@FieldNameConstants
@FieldDefaults(level = AccessLevel.PRIVATE)
@Entity
@Table(name = "BS_LOCATION_FENCE_RULE", indexes = {})
@Comment("围栏规则")
public class LocationFenceRule implements Serializable {

    private static final long serialVersionUID = -6683465582430417205L;

    @Comment("主键")
    @Id
    @GeneratedValue(generator = SnowflakeId.GENERATOR)
    @GenericGenerator(name = SnowflakeId.GENERATOR, strategy = SnowflakeId.Strategy.LONG)
    Long id;

    @Comment("围栏")
    @ManyToOne
    @JoinColumn(name = "fence_id")
    LocationFence fence;

    @FieldMapping("entityTypes")
    @Comment("规则模型")
    @Enumerated(EnumType.STRING)
    Mode mode;

    @Comment("规则名称")
    String name;

    @Comment("规则描述")
    String describe;

    @Comment("最小值（人数）")
    Integer minValue;

    @Comment("最大值（人数）")
    Integer maxValue;

    @Comment("规则生效开始时间")
    LocalTime minTime;

    @Comment("规则生效结束时间")
    LocalTime maxTime;

    @Comment("允许的用户（人员）")
    @OneToMany
    @Builder.Default
    @ToString.Exclude
    Set<User> includes = new HashSet<>(0);

    @Comment("不允许的用户（人员）")
    @OneToMany
    @Builder.Default
    @ToString.Exclude
    Set<User> excludes = new HashSet<>(0);

    @SuppressWarnings({ "unused" })
    public enum Mode {
        STAFF("人员"),
        NUMBER("数量");

        @Getter
        final String text;

        Mode(String text) {
            this.text = text;
        }
    }
}
